"""
nginx-set-conf - A tool for managing Nginx configurations
"""

__version__ = "1.9.1"

from . import config_templates as config_templates
from . import utils as utils
from . import validators as validators
